@@ -1,7 +1,6 @@
 from abc import ABC
 from typing import Union
 import numpy as np
-import pandas as pd
 import ast
 import sys
 from time import time
@@ -15,9 +14,10 @@ class Activation(ABC):
 
     DTYPE = str
     FORCE_STAT = False
+    WILL_COMPARE = False
 
     def __init__(
-        self, activation: Union[np.ndarray, bitarray, str] = None, optimize: bool = True,
+        self, activation: Union[np.ndarray, bitarray, str, int] = None, optimize: bool = True, length: int = None
     ):
         """Compresses an activation vector into a str(list) describing its variations or an bitarray of booleans
 
@@ -28,21 +28,31 @@ class Activation(ABC):
             same as str(list) but the list is casted into np.array instead of str
         or an bitarray:
             The input vector [1 0 0 1 0 0 0 1 1...] where each entry uses up one bit of memory
+        or an integer
+            taking the input vector [1 0 0 1 0 0 0 1 1...], converts it to binary string representation :
+            "100100011..." then cast it into int using int(s, 2). This is done if Activation.WILL_COMPARE is True :
+            converting to int is slower than to bit array, but comparing ints is faster. Size is equivalent to bitarray.
 
         The method will choose how to store the data based on the size (in MB) of the compressed list : if it is
-        superior size in bitarray, it will take less memory and is prefered. If compression is used and
+        superior than in bitarray/int, it will take less memory and is prefered. If compression is used and
         dtype is np.ndarray, will check that numbers present in the compressed vector can be stored as int32 to gain
         memory. Else, uses int64.
 
         Parameters
         ----------
-        activation: Union[np.ndarray, bitarray, str]
+        activation: Union[np.ndarray, bitarray, str, int]
             If np.ndarray : Of the form [0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1], or compressed vector
             If str : compressed vector
             If bitarray : same as np.array but takes 64x less memory (each entry is stored in one bit only)
+            If int : as an integer
         optimize: bool
-            Only relevent 'value' is an integer. In that case, will check whether using compression saves up memory.
-            Else, does not check and uses integer. Note that is optimize is True, entropy is computed.
+            Only relevent 'value' is an bitarray or integer. In that case, will check whether using compression saves up
+            memory. Else, does not check and uses bitarray or integer. Note that if optimize is True, entropy is
+            computed.
+        length: int
+            Only valid if 'value' is an integer. An activation vector stored as an integer has lost the information
+            about its size : [0 0 0 1 0 0 0 1 1...] to nit gives 100011... which in turn gives back [1 0 0 0 1 1...].
+            To get the leading zeros back, one must specify the length of the activation vector.
         """
         self.length = None  # Will be set by init methods
         self._entropy = None  # Will be set if activation is not an integer or if optimize is True
@@ -54,43 +64,62 @@ class Activation(ABC):
         self._coverage = None
         self._time_compressed_to_raw = -1
         self._time_raw_to_compressed = -1
+        self._time_raw_to_integer = -1
         self._time_bitarray_to_raw = -1
         self._time_raw_to_bitarray = -1
         self._time_compressed_to_bitarray = -1
         self._time_bitarray_to_compressed = -1
+        self._time_integer_to_compressed = -1
+        self._time_integer_to_raw = -1
         self._n_compressed_to_raw = 0
         self._n_raw_to_compressed = 0
         self._n_bitarray_to_raw = 0
+        self._n_integer_to_raw = 0
         self._n_raw_to_bitarray = 0
+        self._n_raw_to_integer = 0
         self._n_bitarray_to_compressed = 0
+        self._n_integer_to_compressed = 0
         self._n_compressed_to_bitarray = 0
+        self._n_compressed_to_integer = 0
         self._sizeof_compressed_array = -1
         self._sizeof_compressed_str = -1
         self._sizeof_bitarray = -1
+        self._sizeof_integer = -1
         self._sizeof_raw = -1
 
         if isinstance(activation, str) and "," not in activation:
-            activation = bitarray(activation)
+            if Activation.WILL_COMPARE:
+                activation = int(activation, 2)
+            else:
+                activation = bitarray(activation)
 
         if isinstance(activation, bitarray):
             self._init_with_bitarray(activation, Activation.DTYPE, optimize)
 
+        elif isinstance(activation, int):
+            self._init_with_integer(activation, Activation.DTYPE, length, optimize)
+
         elif isinstance(activation, str):
             self._init_with_str(activation)
+
         elif isinstance(activation, np.ndarray):
             if activation[-1] > 1:
                 self._init_with_compressed_array(activation)
             else:
                 self._init_with_raw(activation, Activation.DTYPE)
         else:
-            raise TypeError(f"An activation can only be a np.ndarray, and bitarray or a str. Got {type(activation)}.")
+            raise TypeError(
+                f"An activation can only be a np.ndarray, and bitarray, a str or an integer. Got"
+                f" {type(activation)}."
+            )
 
     def _init_with_bitarray(self, value: bitarray, dtype: type, optimize: bool = True):
 
         """
         Will set
+          * self._nones (number of ones in the activation)
+          * self._ones (index of ones)
             if Optimize is True:
-              * self._nones (number of ones in the activation)
               * self.length
               * self._entropy and self._rel_entropy
               * self.data_format to "bitarray" or "compressed_str" or "compressed_array" depending on what takes less
@@ -105,13 +134,14 @@ class Activation(ABC):
         logger.debug(f"Activation vector is a bitarray")
         self.length = len(value)
         self._sizeof_bitarray = sys.getsizeof(value) / 1e6
+        self._nones = value.count(1)
+        self._ones = value.search(1)
 
         if optimize:
-            self._nones = value.count(1)
             t0 = time()
             compressed = self._compress(value, dtype=dtype)
-            self._time_raw_to_compressed = time() - t0
-            self._n_raw_to_compressed += 1
+            self._time_bitarray_to_compressed = time() - t0
+            self._n_bitarray_to_compressed += 1
             if isinstance(compressed, str):
                 self._sizeof_compressed_str = sys.getsizeof(compressed) / 1e6
                 size_compressed = self._sizeof_compressed_str
@@ -136,6 +166,63 @@ class Activation(ABC):
             self.data = value
             self.data_format = "bitarray"
 
+    def _init_with_integer(self, value: int, dtype: type, length: int = None, optimize: bool = True):
+
+        """
+        Will set
+            if Optimize is True:
+              * self._nones (number of ones in the activation)
+              * self._ones (index of ones)
+              * self.length
+              * if optimize is True : self._entropy and self._rel_entropy
+              * self.data_format to "integer" or "compressed_str" or "compressed_array" depending on what takes less
+                memory
+              * self.data as an integer, a str or an array
+            else:
+              * self.data as an integer
+              * self.data_format to "integer"
+        """
+
+        if length is None:
+            raise ValueError("When giving an integer to Activation, you must also specify its length.")
+
+        logger.debug(f"Activation vector is an int")
+        self.length = length
+        self._sizeof_integer = sys.getsizeof(value) / 1e6
+
+        if optimize:
+            raw = self._integer_to_raw(value)
+            self._sizeof_raw = sys.getsizeof(raw) / 1e6
+            self._nones = np.count_nonzero(raw == 1)
+            self._ones = np.where(raw == 1)[0].tolist()
+            t0 = time()
+            compressed = self._compress(raw, dtype=dtype)
+            self._time_integer_to_compressed = time() - t0
+            self._n_integer_to_compressed += 1
+            if isinstance(compressed, str):
+                self._sizeof_compressed_str = sys.getsizeof(compressed) / 1e6
+                size_compressed = self._sizeof_compressed_str
+            else:
+                self._sizeof_compressed_array = sys.getsizeof(compressed) / 1e6
+                size_compressed = self._sizeof_compressed_array
+            if dtype == str:
+                self._entropy = len(ast.literal_eval(compressed)) - 2
+            else:
+                self._entropy = len(compressed) - 2
+            self._rel_entropy = self._entropy / self.length
+            if size_compressed > self._sizeof_integer:
+                self.data = value
+                self.data_format = "integer"
+            else:
+                self.data = compressed
+                if dtype == str:
+                    self.data_format = "compressed_str"
+                else:
+                    self.data_format = "compressed_array"
+        else:
+            self.data = value
+            self.data_format = "integer"
+
     def _init_with_str(self, value: str):
         """
         will set :
@@ -177,11 +264,15 @@ class Activation(ABC):
           * self._entropy and self._rel_entropy
           * self.length
           * self._nones
+          * self._ones
         """
+        if value.dtype != np.ubyte:
+            value = value.astype(np.ubyte)
         logger.debug(f"Activation vector is raw")
         self._sizeof_raw = sys.getsizeof(value) / 1e6
         self.length = len(value)
         self._nones = np.count_nonzero(value == 1)
+        self._ones = np.where(value == 1)[0].tolist()
         t0 = time()
         compressed = self._compress(value, dtype=dtype)
         self._time_raw_to_compressed = time() - t0
@@ -218,7 +309,9 @@ class Activation(ABC):
         if self.length != other.length:
             raise ValueError(f"Activations have different lengths. Left is {self.length}, right is {other.length}")
 
-        if self.data_format == "bitarray" and other.data_format == "bitarray":
+        if (self.data_format == "bitarray" and other.data_format == "bitarray") or (
+            self.data_format == "integer" and other.data_format == "integer"
+        ):
             return Activation(self.data & other.data)
         else:
             return Activation(self.raw * other.raw)
@@ -226,7 +319,9 @@ class Activation(ABC):
     def __or__(self, other: "Activation") -> "Activation":
         if self.length != other.length:
             raise ValueError(f"Activations have different lengths. Left is {self.length}, right is {other.length}")
-        if self.data_format == "bitarray" and other.data_format == "bitarray":
+        if (self.data_format == "bitarray" and other.data_format == "bitarray") or (
+            self.data_format == "integer" and other.data_format == "integer"
+        ):
             return Activation(self.data or other.data)
         else:
             return Activation(np.logical_or(self.raw, other.raw, length=self.length).astype("int32"))
@@ -234,7 +329,9 @@ class Activation(ABC):
     def __add__(self, other: "Activation") -> "Activation":
         if self.length != other.length:
             raise ValueError(f"Activations have different lengths. Left is {self.length}, right is {other.length}")
-        if self.data_format == "bitarray" and other.data_format == "bitarray":
+        if (self.data_format == "bitarray" and other.data_format == "bitarray") or (
+            self.data_format == "integer" and other.data_format == "integer"
+        ):
             val_xor = self.data ^ other.data
             val_and = self.data & other.data
             val = val_xor ^ val_and
@@ -247,7 +344,9 @@ class Activation(ABC):
     def __sub__(self, other: "Activation") -> "Activation":
         if self.length != other.length:
             raise ValueError(f"Activations have different lengths. Left is {self.length}, right is {other.length}")
-        if self.data_format == "bitarray" and other.data_format == "bitarray":
+        if (self.data_format == "bitarray" and other.data_format == "bitarray") or (
+            self.data_format == "integer" and other.data_format == "integer"
+        ):
             return Activation((self.data ^ other.data) & self.data)
         else:
             return Activation(np.logical_xor(self.raw, other.raw).astype("int32") * self.raw)
@@ -255,6 +354,45 @@ class Activation(ABC):
     def __len__(self):
         return self.length
 
+    def _integer_to_raw(self, value: int = None, out: bool = True) -> np.ndarray:
+        """From a value of the form 45786542 (int), which is the base 10 representation of the binary form of an
+        activation vector, returns the initial vector.
+        """
+        t0 = time()
+        if value is None:
+            out = False
+            value = self.data
+        if isinstance(value, np.ndarray) and (value[-1] == 0 or value[-1] == 1):
+            if not out:
+                self._time_integer_to_raw = time() - t0
+                self._n_integer_to_raw += 1
+            return value
+        elif not isinstance(value, int):
+            raise TypeError("Can not apply _integer_to_raw on a compressed vector or bitarray")
+        act = np.fromiter(bin(value)[2:], dtype=np.ubyte)
+        if self._sizeof_integer == -1 and not out:
+            self._sizeof_integer = sys.getsizeof(value) / 1e6
+
+        if len(act) > self.length:
+            raise ValueError(
+                "After using _integer_to_raw, I ended up with an activation vector bigger than the specified "
+                "max length. This should not happend as the max length should have been set by the indexing "
+                "of x earlier in your code"
+            )
+        act_bis = np.zeros(self.length).astype(np.ubyte)
+        act_bis[self.length - len(act):] = act
+
+        if not out:
+            if self._sizeof_raw == -1:
+                self._sizeof_raw = sys.getsizeof(act_bis) / 1e6
+            self._time_integer_to_raw = time() - t0
+            self._n_integer_to_raw += 1
+            if self._nones is None:
+                self._nones = np.count_nonzero(act_bis == 1)
+            if self._ones is None:
+                self._ones = np.where(act_bis == 1)[0].tolist()
+        return act_bis
+
     def _bitarray_to_raw(self, value: bitarray = None, out=True) -> np.ndarray:
         """Transforms a bitarray to a nparray
         """
@@ -268,33 +406,37 @@ class Activation(ABC):
                 self._n_bitarray_to_raw += 1
             return value
         elif not isinstance(value, bitarray):
-            raise ValueError("Can not apply _bitarray_to_raw on a compressed vector")
-        act = np.array(list(value), dtype=np.ushort)
+            raise TypeError("Can not apply _bitarray_to_raw on a compressed vector or integer")
+        act = np.array(list(value), dtype=np.ubyte)
 
-        if self._sizeof_raw == -1 and not out:
-            self._sizeof_raw = sys.getsizeof(act) / 1e6
-        if self._sizeof_bitarray == -1 and not out:
-            self._sizeof_bitarray = sys.getsizeof(value) / 1e6
         if not out:
+            if self._sizeof_raw == -1:
+                self._sizeof_raw = sys.getsizeof(act) / 1e6
+            if self._sizeof_bitarray == -1:
+                self._sizeof_bitarray = sys.getsizeof(value) / 1e6
             self._time_bitarray_to_raw = time() - t0
             self._n_bitarray_to_raw += 1
         return act
 
     def _decompress(self, value: Union[str, np.ndarray] = None, raw=True, out=True) -> Union[np.ndarray, bitarray]:
-        """Will return the original activation vector, and set self._nones and self._ones"""
+        """Will return the original activation vector, and set self._nones and self._ones
+
+        If raw is True (default), returns it as a np.ndarray, else as a bitarray
+        """
         t0 = time()
 
         if value is None:
             out = False
             value = self.data
 
+        if not isinstance(value, (str, np.ndarray)):
+            raise TypeError("Can not apply _decompress on a bitarray or integer vector")
+
         if value[-1] == 0 or value[-1] == 1:
             if not out:
                 self._time_compressed_to_raw = time() - t0
                 self._n_compressed_to_raw += 1
             return value
-        elif isinstance(value, bitarray):
-            raise ValueError("Can not apply _decompress on a bitarray vector")
 
         if isinstance(value, str):
             act = ast.literal_eval(value)
@@ -308,33 +450,18 @@ class Activation(ABC):
             raise TypeError(f"'value' can not be of type {type(value)}")
 
         length = act[-1]
-        s = np.zeros(length, dtype=np.ushort)
-        ones = []
-        n_ones = 0
+        s = np.zeros(length, dtype=np.ubyte)
         previous_value = 0
         previous_index = 0
 
-        compute_nones = self._nones is None and not out
-        compute_ones = self._ones is None and not out
-
         if act[0] == 1:
             previous_value = 1
             s[0] = 1
         if len(act) == 2:
             if act[0] == 1:
-                if compute_nones:
-                    self._nones = 1
-                if compute_ones:
-                    self._ones = [pd.IndexSlice[0:1]]
-                s = np.ones(length, dtype=np.ushort)
-            else:
-                if compute_nones:
-                    self._nones = 0
-                if compute_ones:
-                    self._ones = []
-
+                s = np.ones(length, dtype=np.ubyte)
             if raw:
-                act = np.array(s, dtype=np.ushort)
+                act = np.array(s, dtype=np.ubyte)
             else:
                 # noinspection PyTypeChecker
                 act = bitarray(s.tolist())
@@ -342,6 +469,9 @@ class Activation(ABC):
             if not out:
                 if self._sizeof_raw == -1:
                     self._sizeof_raw = sys.getsizeof(act) / 1e6
+                if self._ones is None:
+                    self._ones = np.where(act == 1)[0].tolist()
+                    self._nones = np.count_nonzero(act == 1)
                 self._time_compressed_to_raw = time() - t0
                 self._n_compressed_to_raw += 1
             return act
@@ -351,22 +481,12 @@ class Activation(ABC):
                 previous_index = index
                 previous_value = 1
             else:
-                if compute_nones:
-                    n_ones += index - previous_index
-                if compute_ones:
-                    ones.append(pd.IndexSlice[previous_index:index])
                 s[previous_index:index] = np.ones(index - previous_index)
                 previous_index = index
                 previous_value = 0
 
-        if compute_nones:
-            self._nones = n_ones
-            self._coverage = self._nones / self.length
-
-        if compute_ones:
-            self._ones = ones
         if raw:
-            act = np.array(s, dtype=np.ushort)
+            act = np.array(s, dtype=np.ubyte)
             if not out:
                 if self._sizeof_raw == -1:
                     self._sizeof_raw = sys.getsizeof(act) / 1e6
@@ -380,6 +500,9 @@ class Activation(ABC):
                     self._sizeof_bitarray = sys.getsizeof(act) / 1e6
                 self._time_compressed_to_bitarray = time() - t0
                 self._n_compressed_to_bitarray += 1
+        if self._ones is None:
+            self._ones = np.where(act == 1)[0].tolist()
+            self._nones = np.count_nonzero(act == 1)
         return act
 
     def __contains__(self, other: "Activation") -> bool:
@@ -401,6 +524,8 @@ class Activation(ABC):
         The compressed vector can be stored as a str looking like "0, 12, 456, ..., 47782" or as a numpy array of
         integers. What storage to use is specified by the "dtype" argument.
         """
+        if isinstance(value, int):
+            raise TypeError("Can not compress an integer vector")
         if not isinstance(value, (np.ndarray, bitarray)) or (value[-1] != 0 and value[-1] != 1):
             if isinstance(value, str):
                 return np.array(value.split(",")).astype(int)
@@ -429,15 +554,37 @@ class Activation(ABC):
         if isinstance(value, bitarray):
             return value
         elif not isinstance(value, np.ndarray) or (value[-1] != 0 and value[-1] != 1):
-            raise ValueError("Can not use _array_to_bitarray or a compressed vector")
+            raise TypeError("Can not use _raw_to_bitarray on a compressed vector")
+        elif isinstance(value, int):
+            raise TypeError("Can not use _raw_to_bitarray on a integer vector")
         # noinspection PyTypeChecker
         return bitarray(value.tolist())
 
+    @staticmethod
+    def _raw_to_integer(value: np.ndarray) -> int:
+        """Casts a raw activation vector into the integer represented by its binary form
+        Examples
+        --------
+        >>> from ruleskit import Activation
+        >>> Activation._raw_to_integer(np.array([0, 1, 1, 0]))
+        6  # the binary number '0110' is 6 in base 10
+        """
+        if isinstance(value, int):
+            return value
+        elif not isinstance(value, np.ndarray) or (value[-1] != 0 and value[-1] != 1):
+            raise TypeError("Can not use _raw_to_integer or a compressed vector")
+        elif isinstance(value, bitarray):
+            raise TypeError("Can not use _raw_to_integer on a bitarray vector")
+        to_ret = int("".join(str(i) for i in value.astype(np.ubyte)), 2)
+        return to_ret
+
     @property
     def raw(self) -> np.ndarray:
         """Returns the raw np.array. Will set relevant sizes if this has not been done yet"""
         if self.data_format == "bitarray":
             return self._bitarray_to_raw()
+        elif self.data_format == "integer":
+            return self._integer_to_raw()
         else:
             return self._decompress()  # will also set self._ones and self._nones
 
@@ -452,10 +599,7 @@ class Activation(ABC):
     def nones(self) -> int:
         """self._nones might not be set since it can only be set at object creation if the full array was given"""
         if self._nones is None:
-            if self.data_format == "bitarray":
-                self._nones = self.data.count(1)  # faster than calling "raw"
-            else:
-                _ = self.as_bitarray  # calling as_bitarray will compute nones
+            _ = self.raw  # calling raw will compute nones and ones
 
         if self._coverage is None:
             self._coverage = self._nones / self.length
@@ -464,23 +608,23 @@ class Activation(ABC):
     @property
     def entropy(self) -> int:
         if self._entropy is None:
-            if self.data_format == "bitarray":
-                t0 = time()
-                compressed = self._compress(self.data)
+            t0 = time()
+            fmt = self.data_format
+            compressed = self._compress(self.data)
+
+            if fmt == "bitarray":
                 self._time_bitarray_to_compressed = time() - t0
                 self._n_bitarray_to_compressed += 1
-                if self.data_format == "compressed_str":
-                    if self._sizeof_compressed_str == -1:
-                        self._sizeof_compressed_str = sys.getsizeof(compressed) / 1e6
-                if self.data_format == "compressed_array":
-                    if self._sizeof_compressed_array == -1:
-                        self._sizeof_compressed_array = sys.getsizeof(compressed) / 1e6
-                self._entropy = len(ast.literal_eval(compressed)) - 2
             else:
-                raise ValueError(
-                    "Data format is not bitarray and yet entropy is not set. There is a problem in the "
-                    "Activation class, please contact its maintainer."
-                )
+                self._time_integer_to_compressed = time() - t0
+                self._n_integer_to_compressed += 1
+
+            if self.data_format == "compressed_str" and self._sizeof_compressed_str == -1:
+                self._sizeof_compressed_str = sys.getsizeof(compressed) / 1e6
+            if self.data_format == "compressed_array" and self._sizeof_compressed_array == -1:
+                self._sizeof_compressed_array = sys.getsizeof(compressed) / 1e6
+
+            self._entropy = len(ast.literal_eval(compressed)) - 2
         if self._rel_entropy is None:
             self._rel_entropy = self._entropy / self.length
         return self._entropy
@@ -503,16 +647,39 @@ class Activation(ABC):
             if self._sizeof_bitarray == -1:
                 self._sizeof_bitarray = sys.getsizeof(self.data)
             return self.data
+        elif self.data_format == "integer":
+            raw = self.raw
+            t0 = time()
+            to_ret = self._raw_to_bitarray(raw)
+            self._time_raw_to_bitarray = time() - t0
+            self._n_raw_to_bitarray += 1
+            if self._sizeof_bitarray == -1:
+                self._sizeof_bitarray = sys.getsizeof(to_ret)
+            return to_ret
         else:
             t0 = time()
             to_ret = self._decompress(raw=False)
-            self._nones = to_ret.count(1)
             self._time_compressed_to_bitarray = time() - t0
             self._n_compressed_to_bitarray += 1
             if self._sizeof_bitarray == -1:
                 self._sizeof_bitarray = sys.getsizeof(to_ret)
             return to_ret
 
+    @property
+    def as_integer(self):
+        if self.data_format == "integer":
+            if self._sizeof_integer == -1:
+                self._sizeof_integer = sys.getsizeof(self.data)
+            return self.data
+        else:
+            t0 = time()
+            to_ret = self._raw_to_integer(self.raw)
+            self._time_raw_to_integer = time() - t0
+            self._n_raw_to_integer += 1
+            if self._sizeof_integer == -1:
+                self._sizeof_integer = sys.getsizeof(to_ret)
+            return to_ret
+
     @property
     def as_compressed(self):
         if self.data_format == "compressed_array":
@@ -523,7 +690,20 @@ class Activation(ABC):
             if self._sizeof_compressed_str == -1:
                 self._sizeof_compressed_str = sys.getsizeof(self.data)
             return self.data
-        else:
+        elif self.data_format == "integer":
+            raw = self._integer_to_raw()
+            self._ones = np.where(raw == 1)[0].tolist()
+            self._nones = np.count_nonzero(raw == 1)
+            t0 = time()
+            to_ret = self._compress(raw)
+            self._time_raw_to_compressed = time() - t0
+            self._n_raw_to_compressed += 1
+            if self.data_format == "compressed_array" and self._sizeof_compressed_array == -1:
+                self._sizeof_compressed_array = sys.getsizeof(to_ret)
+            elif self.data_format == "compressed_str" and self._sizeof_compressed_str == -1:
+                self._sizeof_compressed_str = sys.getsizeof(self.data)
+            return to_ret
+        elif self.data_format == "bitarray":
             t0 = time()
             to_ret = self._compress(self.data)
             self._time_bitarray_to_compressed = time() - t0
@@ -540,6 +720,17 @@ class Activation(ABC):
             if self._sizeof_compressed_array == -1:
                 self._sizeof_compressed_array = sys.getsizeof(self.data)
             return self.data
+        elif self.data_format == "integer":
+            raw = self._integer_to_raw()
+            self._ones = np.where(raw == 1)[0].tolist()
+            self._nones = np.count_nonzero(raw == 1)
+            t0 = time()
+            to_ret = self._compress(raw, dtype=np.ndarray)
+            self._time_raw_to_compressed = time() - t0
+            self._n_raw_to_compressed += 1
+            if self._sizeof_compressed_array == -1:
+                self._sizeof_compressed_array = sys.getsizeof(to_ret)
+            return to_ret
         else:
             t0 = time()
             to_ret = self._compress(self.data, dtype=np.ndarray)
@@ -555,6 +746,17 @@ class Activation(ABC):
             if self._sizeof_compressed_str == -1:
                 self._sizeof_compressed_str = sys.getsizeof(self.data)
             return self.data
+        elif self.data_format == "integer":
+            raw = self._integer_to_raw()
+            self._ones = np.where(raw == 1)[0].tolist()
+            self._nones = np.count_nonzero(raw == 1)
+            t0 = time()
+            to_ret = self._compress(raw, dtype=str)
+            self._time_raw_to_compressed = time() - t0
+            self._n_raw_to_compressed += 1
+            if self._sizeof_compressed_str == -1:
+                self._sizeof_compressed_str = sys.getsizeof(to_ret)
+            return to_ret
         else:
             t0 = time()
             to_ret = self._compress(self.data, dtype=str)
@@ -568,32 +770,30 @@ class Activation(ABC):
     def sizeof_raw(self):
         if self._sizeof_raw == -1 and Activation.FORCE_STAT:
             _ = self.raw
-            if self._sizeof_raw == -1:
-                raise ValueError("Calling 'raw' should have set _sizeof_raw")
         return self._sizeof_raw
 
     @property
     def sizeof_bitarray(self):
         if self._sizeof_bitarray == -1 and Activation.FORCE_STAT:
             _ = self.as_bitarray
-            if self._sizeof_bitarray == -1:
-                raise ValueError("Calling 'as_bitarray' should have set _sizeof_bitarray")
         return self._sizeof_bitarray
 
+    @property
+    def sizeof_integer(self):
+        if self._sizeof_integer == -1 and Activation.FORCE_STAT:
+            _ = self.as_integer
+        return self._sizeof_integer
+
     @property
     def sizeof_compressed_array(self):
         if self._sizeof_compressed_array == -1 and Activation.FORCE_STAT:
             _ = self.as_compressed_array
-            if self._sizeof_compressed_array == -1:
-                raise ValueError("Calling 'as_compressed_array' should have set _sizeof_compressed_array")
         return self._sizeof_compressed_array
 
     @property
     def sizeof_compressed_str(self):
         if self._sizeof_compressed_str == -1 and Activation.FORCE_STAT:
             _ = self.as_compressed_str
-            if self._sizeof_compressed_str == -1:
-                raise ValueError("Calling 'as_compressed_str' should have set _sizeof_compressed_str")
         return self._sizeof_compressed_str
 
     @property
@@ -605,6 +805,15 @@ class Activation(ABC):
             self._n_raw_to_compressed += 1
         return self._time_raw_to_compressed
 
+    @property
+    def time_raw_to_integer(self):
+        if self._time_raw_to_integer == -1 and Activation.FORCE_STAT:
+            t0 = time()
+            _ = self._raw_to_integer(self.raw)
+            self._time_raw_to_integer = time() - t0
+            self._n_raw_to_integer += 1
+        return self._time_raw_to_integer
+
     @property
     def time_raw_to_bitarray(self):
         if self._time_raw_to_bitarray == -1 and Activation.FORCE_STAT:
@@ -612,43 +821,52 @@ class Activation(ABC):
             _ = self._raw_to_bitarray(self.raw)
             self._time_raw_to_bitarray = time() - t0
             self._n_raw_to_bitarray += 1
-            if self._time_raw_to_bitarray == -1:
-                raise ValueError("Calling '_raw_to_bitarray' should have set _time_raw_to_bitarray")
         return self._time_raw_to_bitarray
 
     @property
     def time_compressed_to_raw(self):
         if self._time_compressed_to_raw == -1 and Activation.FORCE_STAT:
             _ = self._decompress(self.as_compressed, out=False)
-            if self._time_compressed_to_raw == -1:
-                raise ValueError("Calling '_decompress' should have set _time_compressed_to_raw")
         return self._time_compressed_to_raw
 
     @property
     def time_bitarray_to_raw(self):
         if self._time_bitarray_to_raw == -1 and Activation.FORCE_STAT:
             _ = self._bitarray_to_raw(self.as_bitarray, out=False)
-            if self._time_bitarray_to_raw == -1:
-                raise ValueError("Calling '_bitarray_to_raw' should have set _time_bitarray_to_raw")
         return self._time_bitarray_to_raw
 
+    @property
+    def time_integer_to_raw(self):
+        if self._time_integer_to_raw == -1 and Activation.FORCE_STAT:
+            _ = self._integer_to_raw(self.as_integer, out=False)
+        return self._time_integer_to_raw
+
     @property
     def time_compressed_to_bitarray(self):
         if self._time_compressed_to_bitarray == -1 and Activation.FORCE_STAT:
             _ = self._decompress(self.as_bitarray, raw=False)
-            if self._time_compressed_to_bitarray == -1:
-                raise ValueError("Calling '_decompress' with raw=False should have set _time_compressed_to_bitarray")
         return self._time_compressed_to_bitarray
 
     @property
-    def time_bitarray_to_compress(self):
+    def time_bitarray_to_compressed(self):
         if self._time_bitarray_to_compressed == -1 and Activation.FORCE_STAT:
+            b = self.as_bitarray
             t0 = time()
-            _ = self._compress(self.as_bitarray)
+            _ = self._compress(b)
             self._time_bitarray_to_compressed = time() - t0
             self._n_bitarray_to_compressed += 1
         return self._time_bitarray_to_compressed
 
+    @property
+    def time_integer_to_compressed(self):
+        if self._time_integer_to_compressed == -1 and Activation.FORCE_STAT:
+            i = self.as_integer
+            t0 = time()
+            _ = self._compress(i)
+            self._time_integer_to_compressed = time() - t0
+            self._n_integer_to_compressed += 1
+        return self._time_integer_to_compressed
+
     @property
     def n_raw_to_compressed(self):
         return self._n_raw_to_compressed
@@ -661,14 +879,30 @@ class Activation(ABC):
     def n_raw_to_bitarray(self):
         return self._n_raw_to_bitarray
 
+    @property
+    def n_raw_to_integer(self):
+        return self._n_raw_to_integer
+
     @property
     def n_bitarray_to_raw(self):
         return self._n_bitarray_to_raw
 
+    @property
+    def n_integer_to_raw(self):
+        return self._n_integer_to_raw
+
     @property
     def n_bitarray_to_compressed(self):
         return self._n_bitarray_to_compressed
 
+    @property
+    def n_integer_to_compressed(self):
+        return self._n_integer_to_compressed
+
     @property
     def n_compressed_to_bitarray(self):
         return self._n_compressed_to_bitarray
+
+    @property
+    def n_compressed_to_integer(self):
+        return self._n_compressed_to_integer
